#ifndef float_image_align_H
#define float_image_align_H

/* Tools for translational alignment between float-valued images. */
/* Last edited on 2007-09-07 08:27:23 by stolfi */ 

#include <bool.h>
#include <r2.h>
#include <i2.h>
#include <float_image.h>

typedef double fial_mismatch_t(int n, r2_t p[]); 
  /* Type of a function that evaluates the mismatch for 
     relative placements {p[0..n-1]}. */

void fial_optimal_shifts
  ( int n,
    fial_mismatch_t *mm, /* Funtion that evaluates the mismatch between the images. */
    r2_t step,           /* Adjustment increments along each axis. */
    i2_t ns,             /* Max number of adjustment steps along each axis. */
    r2_t p[],            /* (IN/OUT) Corresponding points in each image. */
    double *QP           /* (OUT) Mismatch value for best displacement. */
  );
  /* Given {n} images {img[0..n-1]} and {n} points {p[0..n-1]},
    adjusts those points so that the images are as similar as possible
    in the neighborhood of those points.
    
    The adjustment applied to each point {p[i]} along each axis {k} is
    some integer step count {d[i].c[k]} times the given step length
    {step.c[k]}, limited to at most {ns.c[k]} steps in either
    direction. The adjustments are `almost balanced', meaning that the
    sum of the step counts {d[0].c[k]+d[1].c[k]+.. d[n-1].c[k]}, for each
    axis {k}, lies between {-n/2} and {+(n-1)/2}, inclusive.

    The local similarity of the images, for a given set of
    displacements {p[0..n-1]}, is measured by the user-given function
    {mm}. */

/* DEBUGGING */

void fial_debug_displacement(char *label, int n, i2_t d[], r2_t q[], double Q, char *tail);
  /* Prints to {stderr} the (integer) displacement tuple {d[0..n-1]}, the displaced points
   {q[0..n-1]}, and the mismatch value {Q}.  If {d} is NULL, omits it; ditto for {q}. */

#endif
