#ifndef sample_conv_H
#define sample_conv_H

/* {sample_conv.h} - conversion between floating-point and integer samples. */
/* Last edited on 2007-10-10 13:26:59 by stolfi */

#include <stdint.h>
#include <stdlib.h>
#include <stdio.h>

typedef uint32_t sample_uint_t;
  /* A quantized sample value. */

float sample_conv_floatize
  ( sample_uint_t iv, 
    sample_uint_t maxval, 
    double lo,
    double hi, 
    sample_uint_t *imin, 
    sample_uint_t *imax, 
    float *vmin,
    float *vmax
  );
  /* Converts an integer sample {iv} to float, mapping 
    affinely {[0..maxval+1]} to {[lo _ hi]}.  Also updates 
    the range {imin,imax} to enclose the input value {iv}, and 
    the range {vmin,vmax} to enclose the output value {fv}. */
    
void sample_conv_print_floatize_stats
  ( int iChan,           /* Channel index in input image. */
    int oChan,           /* Channel index in output image. */
    sample_uint_t imin,       /* Minimum integer sample seen. */
    sample_uint_t imax,       /* Maximum integer sample seen. */
    sample_uint_t maxval,     /* Maximum possible integer sample. */
    double lo,           /* Low end of float scaling range. */
    double hi,           /* High end of float scaling range. */
    float vmin,          /* Minimum float sample seen. */
    float vmax           /* Maximum float sample seen. */
  );
  /* Prints statistics for floatizing channel {iChan} of a PGM/PPM image
    into channel {oChan} of a float image. */

sample_uint_t sample_conv_quantize
  ( float fv, 
    sample_uint_t maxval, 
    double lo,
    double hi, 
    float *vmin,
    float *vmax, 
    int *clo,
    int *chi,
    sample_uint_t *imin, 
    sample_uint_t *imax
  );
  /* Converts a float sample {fv} to integer, mapping affinely 
    {[lo _ hi]} to {[0..maxval + 1)} with clipping. Also updates the range 
    {vmin,vmax} to enclose the input value {fv}, and the
    range {imin,imax} to enclose the output value {iv}. Increments 
    {clo} or {chi} if the input value was below {lo} or above {li}, 
    respectively. */

void sample_conv_print_quantize_stats
  ( int iChan,           /* Channel index in input image. */
    int oChan,           /* Channel index in output image. */
    float vmin,          /* Minimum float sample seen. */
    float vmax,          /* Maximum float sample seen. */
    double lo,           /* Low end of float scaling range. */
    double hi,           /* High end of float scaling range. */
    int clo,             /* Number of samples seen below {lo}. */
    int chi,             /* Number of samples seen above {hi}. */
    sample_uint_t maxval,     /* Maximum possible integer sample. */
    sample_uint_t imin,       /* Minimum integer sample seen. */
    sample_uint_t imax        /* Maximum integer sample seen. */
  );
  /* Prints statistics for quantizing channel {iChan} of a {sample_conv_t}
    into channel {oChan} of a PGM/PPM image. */

#endif
