#define PROG_NAME "test_align"
#define PROG_DESC "test of {float_image_align.h}"
#define PROG_VERS "1.0"

/* Last edited on 2007-08-08 23:19:56 by stolfi */ 
/* Created on 2007-07-11 by J. Stolfi, UNICAMP */

#define test_align_COPYRIGHT \
  "Copyright  2007  by the State University of Campinas (UNICAMP)"

#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <stdint.h>
#include <string.h>
#include <math.h>

#include <float_image.h>
#include <float_pnm_image.h>
#include <float_image_align.h>
#include <r2.h>
#include <r2x2.h>
#include <bool.h>
#include <jsfile.h>
#include <affirm.h>

int main(int argn, char **argv);
void do_test_align(int n, float_image_t *img[], int NC, int NX, int NY);

float_image_t *read_image
  ( char *prefix, 
    int k,
    char *ext,
    int *colsP, 
    int *rowsP, 
    int *chnsP
  );
  /* Reads a PBM/PGM/PPM image file with name "{prefix}-{KK}.{EXT}", 
    where {KK} is the two-digit value of {k}, and converts it to a float
    image with samples in the range [0_1]. If {*colsP,*rowsP,*chnsP} are
    negative, sets them to the image's dimensions; otherwise requires
    that they match the image's dimensions. If {verbose} is true, 
    prints image statistics to {stderr}. */

#define NMAX 5
  /* Max number of images for simultaneous alignment. */

int main(int argn, char **argv)
  {
    /* Input images for testing: */
    char *ext = "ppm"; /* Fixed for now. */
    int NX = -1; /* Number of columns. */
    int NY = -1; /* Number of rows. */
    int NC = -1; /* Number of channels. */
    float_image_t *img[NMAX];
    int k;
    for (k = 0; k < NMAX; k++) 
      { img[k] = read_image("img/test-in", k, ext, &NX, &NY, &NC); }
    
    /* Run test with various image counts: */
    int n;
    for (n = 0; n < NMAX; n++) 
      { do_test_align(n, img, NC, NX, NY); }
    
    return 0;
  }
  
void do_test_align(int n, float_image_t *img[], int NC, int NX, int NY)
  { 
    fprintf(stderr, "testing with %d %d-channel images...\n", n, NC);
    
    r2_t p[n];     /* Corresponding points in each image. */
    double Q = 0;  /* Mismatch of best alignment. */
    int k;
    for (k = 0; k < n; k++) { p[k] = (r2_t){{ 9, 9 }}; }
      
    i2_t NR = (i2_t){{ 5, 7 }}; /* Radius of comparison window (pixels). */
    i2_t NS = (i2_t){{ 3, 4 }}; /* Number of alignment adjustment steps to consider. */
    r2_t step = (r2_t) {{ 0.25, 0.50 }};  /* Adjustment steps along each axis. */
    
    fial_align_var(n, img, NC, NS, step, p, NR, &Q);
    
    fial_debug_displacement("best displacement", n, NULL, p, Q, "\n");
  }

float_image_t *read_image
  ( char *prefix, 
    int k,
    char *ext,
    int *colsP, 
    int *rowsP, 
    int *chnsP
  )
{ char *fname = NULL;
    asprintf(&fname, "%s-%02d.%s", prefix, k, ext);
    pnm_image_t *pim = pnm_image_read(fname);
    float_image_t *fim = float_image_from_pnm_image(pim, NULL, NULL, TRUE);
    if ((*colsP) < 0) { (*colsP) = pim->cols; } else { demand((*colsP) == pim->cols, "bad img"); } 
    if ((*rowsP) < 0) { (*rowsP) = pim->rows; } else { demand((*rowsP) == pim->rows, "bad img"); } 
    if ((*chnsP) < 0) { (*chnsP) = pim->chns; } else { demand((*chnsP) == pim->chns, "bad img"); } 
    pnm_image_free(pim);
    free(fname);
    return fim;
  }

