/* See {indexing_descr.h}. */
/* Last edited on 2007-10-10 20:59:48 by stolfi */

#define indexing_descr_C_COPYRIGHT "Copyright  2003  Jorge Stolfi, State University of Campinas"

#include <indexing_descr.h>
#include <indexing.h>
#include <indexing_io.h>
#include <bool.h>

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <affirm.h>
    
#define NA ix_descr_NAXES
  /* For short. */

ix_descr_t ix_descr_from_sizes ( ix_size_t sz[] )
  { ix_descr_t D;
    D.bp = 0;
    ix_sizes_assign(NA, D.sz, sz);
    ix_steps_from_sizes(NA, D.sz, D.st);
    return D;
  }

ix_pos_t ix_descr_position ( ix_descr_t *D, ix_index_t ix[] )
  { return ix_position(NA, ix, D->bp, D->st); }

void ix_descr_indices_fill ( ix_index_t ix[], ix_index_t val )
  { ix_fill(NA, ix, val); }

void ix_descr_indices_assign ( ix_index_t ix[], ix_index_t *val )
  { ix_assign(NA, ix, val); }

bool_t ix_descr_indices_first ( ix_descr_t *D, ix_index_t ix[] )
  { return ix_assign_min(NA, ix, D->sz); }

bool_t ix_descr_indices_last ( ix_descr_t *D, ix_index_t ix[] )
  { return ix_assign_max(NA, ix, D->sz); }

void ix_descr_indices_shift ( ix_index_t ix[], ix_index_t inc[] )
  { ix_shift(NA, ix, inc); }

bool_t ix_descr_indices_are_valid ( ix_descr_t *D, ix_index_t ix[] )
  { return ix_is_valid(NA, ix, D->sz); }

void ix_descr_sizes_assign ( ix_size_t sza[], ix_size_t szb[] )
  { ix_sizes_assign(NA, sza, szb); }

bool_t ix_descr_sizes_shrink ( ix_size_t sza[], ix_size_t szb[] )
  { return ix_sizes_shrink(NA, sza, szb); }

bool_t ix_descr_sizes_expand ( ix_size_t sza[], ix_size_t szb[] )
  { return ix_sizes_expand(NA, sza, szb); }

ix_size_t ix_descr_max_size ( ix_descr_t *D )
  { return ix_max_size(NA, D->sz); }

ix_size_t ix_descr_min_size ( ix_descr_t *D )
  { return ix_min_size(NA, D->sz); }

bool_t ix_descr_is_empty ( ix_descr_t *D )
  { return ix_is_empty(NA, D->sz); }

ix_pos_count_t ix_descr_num_tuples ( ix_descr_t *D )
  { return ix_num_tuples(NA, D->sz); }

ix_pos_count_t ix_descr_num_positions ( ix_descr_t *D )
  { return ix_num_positions(NA, D->sz, D->st); }

ix_pos_t ix_descr_min_pos ( ix_descr_t *D )
  { return ix_min_pos(NA, D->sz, D->bp, D->st); }

ix_pos_t ix_descr_max_pos ( ix_descr_t *D )
  { return ix_max_pos(NA, D->sz, D->bp, D->st); }

bool_t ix_descr_same_size ( ix_descr_t *A, ix_descr_t *B, bool_t die )
  { return ix_same_size(NA, A->sz, B->sz, die); }

bool_t ix_descr_contained ( ix_descr_t *A, ix_descr_t *B, bool_t die )
  { return ix_contained(NA, A->sz, B->sz, die); }

bool_t ix_descr_next ( ix_descr_t *D, ix_index_t ix[], ix_pos_t *p )
  { return ix_next(NA, ix, D->sz, D->st, p);
  }

bool_t ix_descr_prev ( ix_descr_t *D, ix_index_t ix[], ix_pos_t *p )
  { return ix_prev(NA, ix, D->sz, D->st, p);
  }

sign_t ix_descr_compare ( ix_index_t ixa[], ix_index_t ixb[] )
  { return ix_compare(NA, ixa, ixb); }
    
/* DESCRIPTOR MANIPULATION */

void ix_descr_crop ( ix_descr_t *D, ix_axis_t i, ix_size_t skip, ix_size_t keep )
  { ix_crop(NA, D->sz, &(D->bp), D->st, i, skip, keep); }

void ix_descr_subsample ( ix_descr_t *D, ix_axis_t i, ix_size_t stride )
  { ix_subsample(NA, D->sz, &(D->bp), D->st, i, stride); }

void ix_descr_flip ( ix_descr_t *D, ix_axis_t i )
  { ix_flip(NA, D->sz, &(D->bp), D->st, i); }

void ix_descr_replicate ( ix_descr_t *D, ix_axis_t i, ix_size_t sz )
  { ix_replicate(NA, D->sz, &(D->bp), D->st, i, sz); }

void ix_descr_swap_indices ( ix_descr_t *D, ix_axis_t i, ix_axis_t j )
  { ix_swap_indices(NA, D->sz, &(D->bp), D->st, i, j); }

void ix_descr_flip_indices ( ix_descr_t *D, ix_axis_t i, ix_axis_t j )
  { ix_flip_indices(NA, D->sz, &(D->bp), D->st, i, j); }

void ix_descr_diagonal ( ix_descr_t *D, ix_axis_t i, ix_axis_t j )
  { ix_diagonal(NA, D->sz, &(D->bp), D->st, i, j); }

void ix_descr_chop ( ix_descr_t *D, ix_axis_t i, ix_size_t sz, ix_axis_t j )
  { ix_chop(NA, D->sz, &(D->bp), D->st, i, sz, j); }

/* ELEMENT ENUMERATION */

void ix_descr_enum ( ix_descr_op_t op, ix_descr_t *A, ix_descr_t *B, ix_descr_t *C )
  {
    /* Obtain a non-null operand {X}: */
    ix_descr_t *X = A;
    if (X == NULL) { X = B; }
    if (X == NULL) { X = C; } 

    /* If all three operands are null, there is nothing to do: */
    if (X == NULL) { return; }

    /* Get the intersection {sz} of the domains: */
    ix_size_t sz[NA];
    ix_descr_sizes_assign(sz, X->sz);
    if ((B != NULL) && (B != X)) { ix_descr_sizes_shrink(sz, B->sz); }
    if ((C != NULL) && (C != X)) { ix_descr_sizes_shrink(sz, C->sz); }
      
    /* Get the bases and steps of non-null arguments: */
    ix_pos_t bpA = (A == NULL ? 0 : A->bp);
    ix_step_t *stA = (A == NULL ? NULL : A->st);
    
    ix_pos_t bpB = (B == NULL ? 0 : B->bp);
    ix_step_t *stB = (B == NULL ? NULL : B->st);
    
    ix_pos_t bpC = (C == NULL ? 0 : C->bp);
    ix_step_t *stC = (C == NULL ? NULL : C->st);
    
    ix_enum(op, NA, sz, bpA, stA, bpB, stB, bpC, stC);
  }

bool_t ix_descr_is_valid ( ix_descr_t *D, bool_t die )
  { if (! ix_parms_are_valid(NA, D->sz, D->bp, D->st, die)) { return FALSE; };
    return TRUE;
  }

bool_t ix_descr_positions_are_distinct ( ix_descr_t *D, bool_t die )
  { if (! ix_positions_are_distinct(NA, D->sz, D->st, die)) { return FALSE; };
    return TRUE;
  }

void ix_descr_print_descr ( FILE *wr, char *pre, ix_descr_t *D, int wd, char *suf )
  { /* Compute the effective dimension {d}: */
    ix_dim_t d = NA; 
    while ((d > 0) && (D->sz[d-1] == 1)) { d--; }
    /* Print the indexing parameters: */
    ix_print_parms(wr, pre, d, &(D->bp), D->sz, D->st, wd, suf);
  }
